
import { ExpectedConditions, browser, ElementFinder, $, ElementArrayFinder, protractor, element, by } from 'protractor';
import moment from 'moment';
const EC = protractor.ExpectedConditions;
export class HelpUtility {
    errData: any;

    async waitUntilElementPresenceOf(elem: ElementFinder, timOut: number, errorMsg: string) {
        return await browser.wait(EC.presenceOf(elem), timOut, errorMsg);
    }

    async waitUntiltextToBePresentInElement(elem: ElementFinder, actualText: string) {
        return await browser.wait(EC.textToBePresentInElement(elem, actualText), 20 * 1000);
    }

    public async waitUntilElementClickable(elem: ElementFinder, timeOut: number, errorMsg: string) {
        await browser.wait(EC.elementToBeClickable(elem), timeOut, errorMsg);

    }

    async waitUntilElementVisible(ele: ElementFinder, timOut: number, errorMsg: string) {

        return await browser.wait(protractor.ExpectedConditions.visibilityOf(ele), timOut, errorMsg);
    }

    async waitUntilElementInVisible(elem: ElementFinder) {

        return await browser.wait(EC.invisibilityOf(elem), 12000);
    }

    async setText(elem: ElementFinder, value: string, errorMsg: string, timeOut?: 40000) {

        await this.waitUntilElementVisible(elem, timeOut, errorMsg);
        await this.jsClearText(elem);
        return await elem.sendKeys(value);

    }

    async click(elem: ElementFinder, erroMsg: string, timeOut = 30000): Promise<void> {
        await this.waitUntilElementClickable(elem, timeOut, erroMsg);
        await browser.executeScript("arguments[0].setAttribute('style','border: solid 2px red;');", elem)
        return await elem.click();

    }
    async getAttrValueJS(elem: ElementFinder, erroMsg: string, attr: any, timeOut = 30000): Promise<void> {
        await this.waitUntilElementClickable(elem, timeOut, erroMsg);
        return await browser.executeScript("return arguments[0].'" + attr + "';", elem);
    }
    async getAttrValue(elem: ElementArrayFinder, index: number, erroMsg: string, attr: any, timeOut = 30000): Promise<string> {
        await this.waitUntilElementVisible(elem.first(), timeOut, erroMsg);
        return await elem.get(index).getAttribute(attr);
    }

    async getAttributeValue(elem: ElementFinder, erroMsg: string, attr: any, timeOut = 30000): Promise<string> {
        await this.waitUntilElementVisible(elem, timeOut, erroMsg);
        return await elem.getAttribute(attr);

    }
    async doubleClick(elem: ElementFinder, erroMsg: string, timeOut?: number) {
        await this.waitUntilElementVisible(elem, timeOut, erroMsg);
        await browser.actions().doubleClick(elem).perform();
    }

    async moveMouseToElement(elem: ElementFinder) {
        return await browser.actions().mouseMove(elem).perform();
    }
    async enterText(elem: ElementFinder, text: any, erroMsg: any, timeOut = 30000) {
        await this.waitUntilElementVisible(elem, timeOut, erroMsg);
        return elem.sendKeys(text);

    }
    async clickByVisibleText(optionElements: ElementArrayFinder, textOption: string) {
        await this.waitUntilElementVisible(optionElements.first(), 1000, 'Grp list not found')
        for (let i = 0; i < optionElements.length; i++) {
            const optionText: string = await optionElements.get(i).getText();
            if (optionText.trim() === textOption.trim()) {
                await optionElements.get(i).click();
            }
            else {
                expect(textOption).toEqual('drop down value is not available ' + textOption);
            }
        }
    }
    async clickListOfElement(elem: ElementArrayFinder, val: string) {
        await elem.filter(async (ele) => {
            const txt = await ele.getText();
            return txt.trim() === val;
        }).first().click();
    }


    async selectByIndex(optionElements: ElementArrayFinder, index: number) {
        const ele = optionElements;
        await ele[index].click();
    }
    async jsClick(ele: ElementFinder) {
        await browser.executeScript('arguments[0].click();', ele);
    }

    async clearText(elem: ElementFinder, timeOut: number, erroMsg) {
        await this.waitUntilElementVisible(elem, timeOut, erroMsg);
        return await elem.clear();
    }


    async getText(elem: ElementFinder, errorMsg: string, timeOut = 30000): Promise<string> {

        await this.waitUntilElementVisible(elem, timeOut, errorMsg);
        return await elem.getText();

    }

    async getTextJsExecuter(elem: ElementFinder, errorMsg: string, timeOut = 30000, bool?: boolean) {
        if (bool) {
            await this.waitUntilElementVisible(elem, timeOut, errorMsg);
            return await browser.executeScript("return arguments[0].value;", elem);
        }
        else {
            return await browser.executeScript("return arguments[0].value;", elem);
        }
    }

    async getTextJsExecuterInnerHTML(elem: ElementFinder, errorMsg: string, timeOut = 30000, bool?: boolean) {
        if (bool) {
            await this.waitUntilElementVisible(elem, timeOut, errorMsg);
            return await browser.executeScript("return arguments[0].text;", elem);
        }
        else {
            return await browser.executeScript("return arguments[0].innerHTML;", elem);
        }


    }

    async selectWindow(window: any) {

        const handles = await browser.getAllWindowHandles();
        await browser.switchTo().window(handles[window]);
    }

    async isElementDisplayed(elem: ElementFinder, timeOut: number, errorMsg: string) {
        await this.waitUntilElementVisible(elem, timeOut, errorMsg);
        return await elem.isDisplayed();
    }

    async isElementSelected(elem: ElementFinder) {
        return await elem.isSelected();
    }

    async isElementPresent(elem: ElementFinder) {

        return await elem.isPresent();
    }

    async isElementEnabled(elem: ElementFinder) {
        return await elem.isEnabled();
    }

    async switchToAlert() {


        // await browser.wait(ExpectedConditions.visibilityOf($('')), 8000, 'text box not found');
        //  await browser.wait(ExpectedConditions.alertIsPresent(), 5000, 'alaert found');
        // await browser.switchTo().alert().then(() => { });
    }

    async sendKeysEnter() {
        const enter = await browser.actions().sendKeys(protractor.Key.ENTER);
        await enter.perform();
    }

    async SendKeysTab() {
        const tab = await browser.actions().sendKeys(protractor.Key.TAB);
        await tab.perform();
    }

    async scrollToBottomOfThePage() {
        await browser.executeScript('window.scrollTo(0, document.body.scrollHeight)');
    }

    async waitBrowserTitleEqualsTo(expectedTitle: string) {
        await browser.wait(ExpectedConditions.titleIs(expectedTitle), 10000);
    }

    async isElementNotDisplayed(elem: ElementFinder) {
        let flag = true;
        try {
            flag = await elem.isDisplayed();
        } catch (err) {
            flag = false;
        }
        return flag;
    }

    async slowType(elem: ElementFinder, value: string, delay: number) {
        await elem.click();
        // tslint:disable-next-line:prefer-for-of
        for (let i = 0; i < value.length; i++) {
            await browser.actions().sendKeys(value[i]).perform();
            await browser.sleep(delay);
        }
    }
    async randomChar(length: number) {
        const randomChars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
        let result = '';
        for (let i = 0; i < length; i++) {
            result += randomChars.charAt(Math.floor(Math.random() * randomChars.length));
        }
        return result;
    }
    async randomMobileNumber() {
        const num = Math.floor(100000 + Math.random() * 900000);
        return 5478 + `${num}`
    }
    async getUIFields(locator: ElementArrayFinder, errorMsg?: string, timeOut = 40000): Promise<any[]> {
        await this.waitUntilElementVisible(locator.first(), timeOut, errorMsg);
        const uiFileds: string[] = [];
        for (let i = 0; i < (await locator).length; i++) {
            uiFileds.push(await locator.get(i).getText());
        }
        return uiFileds;

    }
    async setDate(dateFormat: string) {
        return await moment(new Date()).format(dateFormat);
    }
    async clickButton(btnName: string, index?: number) {
        let ele: ElementFinder;
        if (index) {
            ele = element.all(by.xpath(`//button[normalize-space()='${btnName}']`)).get(index);
        } else {
            ele = element(by.xpath(`//button[normalize-space()='${btnName}']`));
        }
        return await this.click(ele, `${btnName} button not found`, 15000);
    }
    async closeToastMsg() {
        const ele = element.all(by.xpath('//button[@role="button"]/span')).first();
        await this.waitUntilElementInVisible(ele);

    }
    async jsClearText(ele: ElementFinder) {
        await browser.executeScript("arguments[0].value = '';", ele)
    }
    async deletTextKeyboard() {
        await browser.actions().sendKeys(protractor.Key.DELETE).perform();
    }
    async toolTipVal(elem: ElementFinder) {
        await this.moveMouseToElement(elem);
    }

    async clickLink(lnkName: any, toast?: boolean) {
        const ele = await element(by.xpath('//a[normalize-space()="L"]'.replace('L', lnkName)));
        if (toast) {
            await this.closeToastMsg();
            await this.click(ele, `${lnkName} not present `);
        }
        else {
            await this.click(ele, `${lnkName} not present `);
        }

    }
    async getColumnValue(ele: ElementArrayFinder, index: number) {
        const elem = ele;
        return await elem.get(index).getText();
    }
    async getHeadings(h1: string, h3: string): Promise<any> {
        const heading: string = '//h1[normalize-space()="Add"]'.replace('Add', h1);
        const subHeading: string = '//h2[normalize-space()="sub"]'.replace('sub', h3);
        const arr: string[] = [];
        const ele = await element(by.xpath(heading));
        const elem = await element(by.xpath(subHeading));
        const heading1 = await this.getText(ele, 'heading not found');
        const headingSub1 = await this.getText(elem, 'heading not found');
        arr.push(heading1, headingSub1);
        return arr;
    }
    async toastMessageText(message: string) {
        const ele = $('div.toast-message');
        await this.waitUntilElementVisible(ele, 5000, 'toast message not found');
        const msg = await this.getText(ele, 'toast msg not found');
        expect(msg).toEqual(message);
    }

    async accessControlPage() {
        const ele = element(by.xpath('//h1[text()="Access Control"]'));
        return this.waitUntilElementVisible(ele, 10000, 'Access control not found');
    }


    async setTextJsExecuter(elem: ElementFinder, text: string) {
        return await browser.executeScript("return arguments[0].value='" + text + "'", elem);
    }

}

